################################################################################
## Copyright 2019-2020 Lawrence Livermore National Security, LLC and other
## DiHydrogen Project Developers. See the top-level LICENSE file for details.
##
## SPDX-License-Identifier: Apache-2.0
################################################################################

include("${CMAKE_CURRENT_LIST_DIR}/DiHydrogenConfigVersion.cmake")

list(APPEND CMAKE_MODULE_PATH
  "${CMAKE_CURRENT_LIST_DIR}"
  "${CMAKE_CURRENT_LIST_DIR}/modules"
  "")

set(H2_VERSION ${PACKAGE_VERSION})

# Verify the dependencies of H2
include(CMakeFindDependencyMacro)

set(H2_HAS_DISTCONV OFF)
set(H2_HAS_ALUMINUM )
set(H2_HAS_OPENMP )
set(H2_HAS_HALF )
set(H2_HAS_CUDA )
set(H2_HAS_ROCM )
set(H2_DISTCONV_HAS_P2P )
set(H2_DISTCONV_HAS_NVSHMEM )

find_dependency(spdlog)

if (H2_HAS_DISTCONV)
  find_dependency(MPI)
endif ()

if (H2_HAS_ALUMINUM)
  set(_BUILD_AL_DIR "")
  set(_MIN_AL_VERSION "")
  find_dependency(Aluminum ${_MIN_AL_VERSION}
    HINTS "${_BUILD_AL_DIR}")
endif ()

if (H2_HAS_OPENMP)
  find_dependency(OpenMP)
endif ()

if (H2_HAS_HALF)
  find_dependency(Half)
endif ()

if (H2_HAS_CUDA)
  include(SetupCUDAToolkit)
  if (H2_DISTCONV_HAS_P2P)
    find_dependency(P2P)
  endif ()
  if (H2_DISTCONV_HAS_NVSHMEM)
    find_dependency(NVSHMEM)
  endif ()
endif ()

if (H2_HAS_ROCM)
  find_dependency(hip)
  find_library(HSA_LIBRARY hsa-runtime64
    HINTS ${ROCM_PATH}/hsa $ENV{ROCM_PATH}/hsa
    PATH_SUFFIXES lib lib64
    DOC "HSA runtime library"
    NO_DEFAULT_PATH)
  find_library(HSA_LIBRARY hsa-runtime64 REQUIRED)
  find_dependency(MIOpen CONFIG)
  find_dependency(hipcub CONFIG)
  find_dependency(rocm_smi CONFIG)
  find_dependency(Roctracer MODULE)
endif ()


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was DiHydrogenConfig.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

# Now actually import the Hydrogen target
set(_TMP_INCLUDE_DIRS "${PACKAGE_PREFIX_DIR}/include")
foreach (_DIR ${_TMP_INCLUDE_DIRS})
  set_and_check(_INCLUDE_DIR "${_DIR}")
  list(APPEND HYDROGEN_INCLUDE_DIRS "${_INCLUDE_DIR}")
endforeach (_DIR "${_TMP_INCLUDE_DIRS}")

set(_TMP_LIBRARY_DIRS "${PACKAGE_PREFIX_DIR}/lib")
foreach (_DIR ${_TMP_LIBRARY_DIRS})
  set_and_check(_LIBRARY_DIR "${_DIR}")
  list(APPEND HYDROGEN_LIBRARY_DIRS "${_LIBRARY_DIR}")
endforeach (_DIR ${_TMP_LIBRARY_DIRS})

set(_H2_LIBRARIES H2Meta;H2Patterns;;H2Core)
if (NOT TARGET H2::H2)
  include(${CMAKE_CURRENT_LIST_DIR}/DiHydrogenTargets.cmake)

  add_library(H2::H2 IMPORTED INTERFACE)
  foreach (lib IN LISTS _H2_LIBRARIES)
    target_link_libraries(H2::H2 INTERFACE H2::${lib})
    string(SUBSTRING "${lib}" 2 -1 _COMP_NAME)
    set(DiHydrogen_${_COMP_NAME}_FOUND TRUE)
    list(APPEND _FOUND_COMPONENTS "${_COMP_NAME}")
  endforeach ()
  list(SORT _FOUND_COMPONENTS)
  message(STATUS "Found DiHydrogen components: ${_FOUND_COMPONENTS}")
endif (NOT TARGET H2::H2)

check_required_components(DiHydrogen)
set(H2_LIBRARIES H2::H2)
